@@ -1,24 +1,24 @@
 /**
  * \file arc4.h
  */
-#ifndef _ARC4_H
-#define _ARC4_H
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+#ifndef XYSSL_ARC4_H
+#define XYSSL_ARC4_H
 
 /**
  * \brief          ARC4 context structure
  */
 typedef struct
 {
-    int m[256];         /*!< permutation table */
-    int x;              /*!< permutation index */
-    int y;              /*!< permutation index */
+    int x;                      /*!< permutation index */
+    int y;                      /*!< permutation index */
+    unsigned char m[256];       /*!< permutation table */
 }
 arc4_context;
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /**
  * \brief          ARC4 key schedule
  *
@@ -42,7 +42,7 @@ void arc4_crypt( arc4_context *ctx, unsigned char *buf, int buflen );
  *
  * \return         0 if successful, or 1 if the test failed
  */
-int arc4_self_test( void );
+int arc4_self_test( int verbose );
 
 #ifdef __cplusplus
 }
